<?php
final class License extends AppModel
{
    private $license_key;
    private $public_key;

    public function updateLicenseKey($license_key)
    {
        try {
            $this->load();
            $this->public_key  = 123456789;
            $this->license_key = 123456789;
            $this->Settings->setSetting('license_public_key', $this->public_key);
            $this->Settings->setSetting('license_key', $this->license_key);
            $this->Settings->setSetting('license_data', $license_data);

        } catch (Exception $e) {
        }
        $this->unload();
        return true;
    }

    public function validate($revalidate = false)
    {
        return true;
    }

    public function fetchLicense()
    {
        try {
            $this->load();
            if ('' === $this->public_key) {
                $this->fetchKey();
            }

            $license_data = $this->getLicenseData();
            if ('' !== $license_data) {
                $this->Settings->setSetting('license_data', $license_data);
            }

            return $license_data;
        } catch (Exception $e) {
        }
        $this->unload();
    }

    public function verify($key)
    {
        if (function_exists('hash_hmac')) {
            try {
                $data = 'bnsa32047@#lsfJS;lk138tAKDHS:Djh!23172907YAKJhrSa;fgh';

                return hash_hmac('sha256', $data, $key);
            } catch (Exception $e) {
            }
        }
    }

    public function getLocalData()
    {
        try {
            $this->load();
            $license_data = $this->Settings->getSetting('license_data');

            return $this->processLicenseData($license_data->value);
        } catch (Exception $e) {
        }
        $this->unload();
    }

    private function load()
    {
        $path_to_phpseclib       = VENDORDIR . 'phpseclib' . DS;
        $signature_mode          = 'sha256';
        $license_hash            = hash_file($signature_mode, MODELDIR . 'license_manager.php');
        $possible_license_hashes = ['41bf4435b5811a0942390316a952d1312b68a9c028ba261da7689952e8197666', '4e3085121ddef4ac8a37b2ec3a8f5cf728f1fcd7225813b2ac7db2590e7a76f0'];
        if (!in_array($license_hash, $possible_license_hashes, true)) {
            throw new Exception('LicenseManager signature is invalid, LicenseManager can not be initialized.');
        }

        $signatures = [$path_to_phpseclib . 'Crypt' . DS . 'AES.php' => ['8cc0f2b61bf99a0cee66cd6a2d5807baa9d00fb5f774a825b6279c332c3df58f', '27586e9f1ab05369d531d5bb710aa419c5f4a42434a409ec7a3e54f212fb06e7'], $path_to_phpseclib . 'Crypt' . DS . 'Rijndael.php' => ['0348b151113b6eb5eb7e11987c42952ad7d83e5c1ff5456ce199ea21c4e7eaee', '8fe5622865040a459dd14185ecef34769961c6886856fb935176230655ff5c7e'], $path_to_phpseclib . 'Crypt' . DS . 'RSA.php' => ['3c264e335730cabd91566a8435d18cd0465149a7b8ce1e5a7ddea78751bc1e5b', 'fb96f9183231c974ac00a8b3876a597171a8c6265ba6817cae9dbb6fcaa7010d'], $path_to_phpseclib . 'Crypt' . DS . 'Hash.php' => ['2d7edbde0f1062b3b6094845275de7081994fffcbffc8f5a4c2922a5608db945', 'f6d237f22caba4b9e9bcf612dc768beb54e29f393233fb008edecbb184d28ed8'], $path_to_phpseclib . 'Math' . DS . 'BigInteger.php' => ['57bb493d13486092bedcf5a35aa1c6452e0c5d758d6d9e525364140d51fc6c87', 'fafa8e242c20d172cf7794de66c88753bde39a86522f16ddd3bc7454450e9851']];
        Loader::loadModels($this, ['LicenseManager' => [$path_to_phpseclib, $signature_mode, $signatures, ROOTWEBDIR], 'Settings']);
        $license_key = $this->Settings->getSetting('license_key');
        $public_key  = $this->Settings->getSetting('license_public_key');
        if ($license_key) {
            $this->license_key = $license_key->value;
        }

        if ($public_key) {
            $this->public_key = $public_key->value;
        }

        $this->setKeys();
    }

    private function setKeys()
    {
    }

    private function unload()
    {
        if (isset($this->LicenseManager)) {
            $this->LicenseManager = null;
        }

        if (isset($this->Settings)) {
            $this->Settings = null;
        }
    }

    private function fetchKey()
    {
    }

    private function getLicenseData()
    {
    }

    private function processLicenseData($license_data)
    {
        $ttl = 1209600;
        // $result                 = $this->LicenseManager->validate($license_data, $ttl);
        $result = [
            'status'       => 'valid',
            'time'         => time() + 999999999,
            'domain'       => '*',
            'ip'           => '*',
            'path'         => '*',
            'comp_allowed' => 1,
            'comp_total'   => 99999999,
        ];

        try {
            if (!isset($this->Companies)) {
                Loader::loadModels($this, ['Companies']);
            }

            if (isset($result['addons'])) {
                foreach ($result['addons'] as $addon) {
                    if (isset($addon['fields']['type']) && 'company' === $addon['fields']['type']) {
                        $result['comp_allowed'] += $addon['qty'];
                    }
                }
            }

            if ($result['comp_allowed'] < $result['comp_total']) {
                $result['status'] = 'company_quota';
            }
        } catch (Exception $e) {
        }

        return $result;
    }

    private function setError($status)
    {
        $error = null;
        switch ($status) {
            case 'invalid_location':
                $error = 'The license is not valid for the installed location.';

                break;
            case 'suspended':
                $error = 'The license is suspended.';

                break;
            case 'expired':
                $error = 'The license has expired.';
                if ('trial-' === substr($this->license_key, 0, 6)) {
                    $error = 'Sorry, a trial has already been issued for this' . ' domain and is no longer valid. To obtain a new ' . 'trial key, please contact sales@blesta.com. ' . "If you'd like to purchase a license, please " . 'visit www.blesta.com.';
                }

                break;
            case 'unknown':
                $error = 'The license key is invalid.';
                if ('trial-' === substr($this->license_key, 0, 6)) {
                    $error = 'Sorry, a trial has already been issued for this ' . 'domain and is no longer valid. To obtain a new trial' . " key, please contact sales@blesta.com. If you'd like" . ' to purchase a license, please visit www.blesta.com.';
                }

                break;
            case 'company_quota':
                $error = 'The license is not valid for all companies in the system.';

                break;
        }
        $this->Input->setErrors([]);
        if ($error) {
            $this->Input->setErrors(['status' => ['invalid' => $error]]);
        }
    }
}
